@@ -25,10 +25,10 @@ pub use parking_lot;
 pub use serde;
 pub use serde_json;
 pub use serde_v8;
-pub use serde_v8::Buffer as ZeroCopyBuf;
 pub use serde_v8::ByteString;
 pub use serde_v8::StringOrBuffer;
 pub use serde_v8::U16String;
+pub use serde_v8::ZeroCopyBuf;
 pub use sourcemap;
 pub use url;
 pub use v8;
